'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hashUnit = exports.envelopeHasher = exports.contentHasher = exports.idHasher = exports.fieldHasher = exports.baseHasher = undefined;

var _fp = require('lodash/fp');

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hash = function hash(data) {
  var text = (0, _fp.isString)(data) ? data : JSON.stringify(data);
  var sha1sum = _crypto2.default.createHash('sha1');
  sha1sum.update(text);
  return sha1sum.digest('hex');
};
// FIXME: This seems to be a bad name choice.
var baseHasher = exports.baseHasher = hash;

// Return a unit with a new hash value set.
var fieldHasher = exports.fieldHasher = (0, _fp.curry)(function (key, fields, unit) {
  // Make sure that the field and values are always in the same order.
  var data = (0, _fp.reduce)(function (memo, field) {
    return (0, _fp.concat)(memo, [field, (0, _fp.get)(field, unit)]);
  }, [], fields);
  return (0, _fp.merge)(unit, (0, _fp.set)(key, hash(data), {}));
});

var idHasher = exports.idHasher = fieldHasher('_lf_id_hash');
var contentHasher = exports.contentHasher = fieldHasher('_lf_content_hash');

var envelopeHasher = exports.envelopeHasher = function envelopeHasher(hs) {
  var predicate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _fp.identity;
  return function (envelope) {
    var data = (0, _fp.map)(function (unit) {
      return hs(unit);
    }, predicate(envelope.data));
    return (0, _fp.merge)(envelope, { data: data });
  };
};

// Hash a list of elements in a unit.
var hashUnit = exports.hashUnit = (0, _fp.curry)(function (elem, hasher, unit) {
  var hashedElems = (0, _fp.reduce)(function (memo, e) {
    return (0, _fp.concat)(memo, hasher(e));
  }, [], unit[elem]);
  return (0, _fp.merge)(unit, (0, _fp.set)(elem, hashedElems, {}));
});

exports.default = {
  baseHasher: baseHasher,
  fieldHasher: fieldHasher,
  idHasher: idHasher,
  contentHasher: contentHasher,
  envelopeHasher: envelopeHasher,
  hashUnit: hashUnit
};