'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadPlugins = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _plugins = require('./plugins');

var _plugins2 = _interopRequireDefault(_plugins);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// eslint-disable-next-line import/no-dynamic-require
var p = require('' + _path2.default.join(process.cwd(), 'package'));

// A plugin can provide data transformations, history hooks and query
// sources. ts => transformations, hs => histories and qs => queries.
/**
 * Load all plugins available for this littlefork installation.
 *
 * @returns {Array.<Object, Object>} A tuple containing two Objects. The first
 * element is a list of transformation plugins, the second contains all
 * query plugins.
 * @example
 * const [transformations, queries] = loadPlugins();
 */
var loadPlugins = exports.loadPlugins = function loadPlugins() {
  var deps = (0, _fp.flow)([_fp.keys, (0, _fp.filter)((0, _fp.startsWith)('littlefork-plugin'))])(p.dependencies);

  return (0, _fp.reduce)(function (_ref, name) {
    var _ref2 = _slicedToArray(_ref, 2),
        ts = _ref2[0],
        qs = _ref2[1];

    // TODO: That only loads local npm packages, but not packages in global.
    var moduleName = process.cwd() + '/node_modules/' + name;
    var pluginName = (0, _fp.flow)([(0, _fp.split)('-'), (0, _fp.drop)(2), (0, _fp.join)('-')])(name);
    var module = {};

    // TODO: Move side effects out of here. Maybe or Either instead??
    try {
      // eslint-disable-next-line global-require, import/no-dynamic-require
      module = require(moduleName);
    } catch (e) {
      console.error('' + e.message);
      console.error(e);
      console.error(name + ' could not be loaded. Did you run \'npm install\'?');
      process.exit(1);
    }

    // Load data plugins and query sources.
    var transformations = (0, _fp.merge)(ts, module.plugins || {});
    var sources = module.query ? (0, _fp.merge)(qs, _defineProperty({}, pluginName, module.source)) : qs;

    return [transformations, sources];
  }, [_plugins2.default, {}], deps);
};

exports.default = { loadPlugins: loadPlugins };