'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runner = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _baconjs = require('baconjs');

var _baconjs2 = _interopRequireDefault(_baconjs);

var _plugin = require('./plugin');

var _query = require('./query');

var _pluginLoader = require('./plugin-loader');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// To make liftA2 more composeable later on, flip the arguments.
var runPlugin = (0, _fp.curry)(function (plugin, env, envelope) {
  return (0, _plugin.liftA2)(plugin, envelope, env);
});

// The following functions provide funtionalities outside of a plugin.
var stats = (0, _fp.curry)(function (name, envelope) {
  var p = (0, _fp.merge)(envelope.stats.p, _defineProperty({}, name, (0, _fp.size)(envelope.data)));
  return (0, _fp.merge)(envelope, { stats: { p: p } });
});

var start = (0, _fp.curry)(function (stream, name, envelope) {
  stream.push({ type: 'plugin_start', ts: (0, _utils.now)(), plugin: name });
  return envelope;
});

var end = (0, _fp.curry)(function (stream, name, envelope) {
  stream.push({ type: 'plugin_end', ts: (0, _utils.now)(), plugin: name });
  return envelope;
});

/**
 * Create a runable littlefork object.
 *
 * Upon calling this function, you receive a runnable littlefork pipeline and
 * an observable stream. The littlefork pipeline is a function that can be
 * called without any arguments. It will return a promise that resolves to the
 * result of the pipeline run. The stream object is used to receive messages
 * during the pipeline run. It's currently mainly used for logging purposes,
 * but can be used for more as well.
 *
 * The stream sends messages with the following types:
 *
 * - `log_info`
 * - `log_debug`
 * - `log_error`
 * - `plugin_start`
 * - `plugin_end`
 *
 * @param {Object} config Configuration for a littlefork run.
 * @param {Array.<String>} queryIds A list of ids to query.
 * @returns {Array.<Function, Observable>} Return a tuple with a function that
 * runs the pipeline and an observable object, that receives messages during the
 * pipeline run. The observable object is BaconJS stream and has the full BaconJS
 * API available.
 * @example
 * const [run, stream] = runner(config, queryIds);
 *
 * stream.onValue(msg => {
 *   switch (msg.type) {
 *     case 'log_info': console.log(msg.msg); break;
 *     // ... other cases ...
 *     default: break;
 *   }
 * });
 *
 * run();
 */
var runner = exports.runner = (0, _fp.curry)(function (cfg, queryIds) {
  var _loadPlugins = (0, _pluginLoader.loadPlugins)(),
      _loadPlugins2 = _slicedToArray(_loadPlugins, 2),
      transformations = _loadPlugins2[0],
      queryLoaders = _loadPlugins2[1];

  var stream = _baconjs2.default.Bus();

  var pipeline = (0, _fp.flow)([(0, _fp.map)(function (p) {
    return transformations[p];
  }), (0, _fp.zip)(cfg.plugins)])(cfg.plugins);

  var log = {
    info: function info(msg) {
      return stream.push({ type: 'log_info', msg: msg });
    },
    error: function error(msg) {
      return stream.push({ type: 'log_error', msg: msg });
    },
    debug: function debug(msg) {
      return stream.push({ type: 'log_debug', msg: msg });
    }
  };

  var run = function run() {
    return (
      // Iterate over the pipeline and run each transformation in sequence.
      _bluebird2.default.resolve((0, _fp.isEmpty)(queryIds) ? [] : (0, _query.loader)(queryIds, queryLoaders)).then(function (queries) {
        return (0, _utils.reduceP)(function (envelope, _ref) {
          var _ref2 = _slicedToArray(_ref, 2),
              name = _ref2[0],
              plugin = _ref2[1];

          return (0, _utils.flowP)([start(stream, name), runPlugin(plugin, { log: log, cfg: cfg }), stats(name), end(stream, name)])(envelope);
        }, { queries: queries, data: [], stats: { launchTime: (0, _utils.now)(), p: {} } }, pipeline);
      }).catch(function (e) {
        return stream.error(e);
      }).tap(function () {
        return stream.end();
      })
    );
  };

  return [run, stream];
});

exports.default = { runner: runner };