'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.promisify = exports.reduceP = exports.mapP = exports.flowP = exports.composeP = exports.call = exports.printf = exports.tap = exports.now = exports.defaultFields = exports.assertEnv = exports.retry = exports.contentHashes = exports.idHashes = undefined;

var _fp = require('lodash/fp');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _util = require('util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var reduceObj = _fp.reduce.convert({ cap: false });

// String -> [{a}] -> [b]
// Given a list of units, return a list of hashes of these units. `hashField`
// is the name of hash field, e.g. `_ls_id_hash`.
var selectByHash = (0, _fp.curry)(function (hashField, units) {
  return (0, _fp.flow)([(0, _fp.map)(function (unit) {
    return unit[hashField];
  }), _fp.uniq])(units);
});

var idHashes = exports.idHashes = selectByHash('_lf_id_hash');
var contentHashes = exports.contentHashes = selectByHash('_lf_content_hash');

var retry = exports.retry = function retry(action) {
  var cfg = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var opts = (0, _fp.merge)({ times: 5, delay: 250, delayModifier: 1.5 }, cfg);

  var resolver = function resolver(times, delay) {
    var result = _bluebird2.default.try(action);
    return times <= 0 ? result : result.catch(function () {
      return _bluebird2.default.delay(delay);
    }).then(function () {
      return resolver(times - 1, delay * opts.delayModifier);
    });
  };

  return resolver(opts.times, opts.delay);
};

var assertEnv = exports.assertEnv = function assertEnv(expected) {
  var existing = (0, _fp.flow)([(0, _fp.pick)(expected), _fp.keys])(process.env);

  if (!(0, _fp.eq)((0, _fp.size)(existing), (0, _fp.size)(expected))) {
    var missingFields = (0, _fp.difference)(expected, existing);
    var fieldsMsg = (0, _fp.map)(_fp.lowerCase, missingFields).join(', ');
    // TODO: Move side effects out of here. Maybe or Either instead??
    // Temporarily disable this, need to replace it in all modules.
    // error(`Missing configuration options: ${fieldsMsg}`);
    // error(`Existing: ${existing}`);
    // error(`Expected: ${expected}`);
    console.error('Missing configuration options: ' + fieldsMsg);
    console.error('Existing: ' + existing);
    console.error('Expected: ' + expected);
    process.exit(1);
  }
};

// we make sure that we have a set of default fields set.
var defaultFields = exports.defaultFields = function defaultFields(unit) {
  var defaults = {
    _lf_visible: true,
    _lf_starred: false,
    _lf_links: [],
    _lf_images: [],
    _lf_pubdates: { fetch: _moment2.default.utc().toISOString() }
  };

  return reduceObj(function (memo, v, k) {
    return memo[k] ? memo : (0, _fp.merge)(memo, _defineProperty({}, k, v));
  }, unit, defaults);
};

var now = exports.now = function now() {
  return _moment2.default.utc().toISOString();
};

var tap = exports.tap = (0, _fp.curry)(function (f, x) {
  f((0, _fp.cloneDeep)(x));
  return x;
});

var printf = exports.printf = function printf(x) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var options = (0, _fp.merge)({ colors: true }, opts);
  var f = (0, _fp.flow)([(0, _fp.partialRight)(_util.inspect, [options]), console.log]);
  return tap(f, x);
};

// Use this combinator in e.g. map or flow.
var call = exports.call = function call(f) {
  return f();
};

// composeP :: (b -> Future c) -> (a -> Future b) -> Future a -> Future c
// composeP :: (b -> Future c) -> (a -> Future b) -> a -> Future c
// This function is overloaded to allow async and non async input.
var composeP = exports.composeP = (0, _fp.curry)(function (f, g, a) {
  return _bluebird2.default.resolve(a).then(g).then(f);
});

var flowP = exports.flowP = function flowP(fs) {
  return function (x) {
    return (0, _fp.reduce)(function (memo, f) {
      return memo.then(f);
    }, _bluebird2.default.resolve(x), fs);
  };
};

var mapP = exports.mapP = (0, _fp.curry)(function (f, xs) {
  return _bluebird2.default.map(xs, f, { concurrency: 1 });
});

var reduceP = exports.reduceP = (0, _fp.curry)(function (f, acc, xs) {
  return _bluebird2.default.reduce(xs, f, acc);
});

var promisify = exports.promisify = function promisify(f) {
  return (0, _fp.flow)([f, _bluebird2.default.resolve]);
};

exports.default = {
  idHashes: idHashes,
  contentHashes: contentHashes,
  retry: retry,
  assertEnv: assertEnv,
  defaultFields: defaultFields,
  now: now,
  tap: tap,
  printf: printf,
  call: call,
  composeP: composeP,
  flowP: flowP,
  mapP: mapP,
  reduceP: reduceP,
  promisify: promisify
};